import uuid
from datetime import datetime

from marshmallow import post_load
from sqlalchemy import or_, func, orm

from core.managers.db_manager import db
from core.managers.log_manager import logger
from shared.schema.collectors_node import CollectorsNodeSchema, CollectorsNodePresentationSchema


class NewCollectorsNodeSchema(CollectorsNodeSchema):
    @post_load
    def make_collectors_node(self, data, **kwargs):
        return CollectorsNode(
            id=data["id"],
            name=data["name"],
            description=data["description"],
            api_url=data["api_url"],
            api_key=data["api_key"],
        )


class CollectorsNode(db.Model):
    id = db.Column(db.String(), primary_key=True)
    name = db.Column(db.String(), unique=True, nullable=False)
    description = db.Column(db.String())

    api_url = db.Column(db.String(), nullable=False)
    api_key = db.Column(db.String(), nullable=False)

    def __init__(self, id, name, description, api_url, api_key):
        self.id = id if id != "" else str(uuid.uuid4())
        self.name = name
        self.description = description
        self.api_url = api_url
        self.api_key = api_key
        self.tag = "mdi-animation-outline"
        self.type = "collector"

    @orm.reconstructor
    def reconstruct(self):
        self.tag = "mdi-animation-outline"
        self.type = "collector"

    @classmethod
    def exists_by_api_key(cls, api_key):
        return db.session.query(db.exists().where(CollectorsNode.api_key == api_key)).scalar()

    @classmethod
    def get_by_api_key(cls, api_key):
        return cls.query.filter_by(api_key=api_key).first()

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(CollectorsNode.name)).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(CollectorsNode.name).like(search_string),
                    func.lower(CollectorsNode.description).like(search_string),
                )
            )

        return query.order_by(db.asc(CollectorsNode.name)).all(), query.count()

    @classmethod
    def get_by_id(cls, id):
        return cls.query.filter_by(id=id).first()

    @classmethod
    def get_first(cls):
        return cls.query.first()

    @classmethod
    def get_json_by_id(cls, id):
        return CollectorsNodeSchema().dump(cls.get_by_id(id))

    @classmethod
    def get_all_json(cls, search):
        nodes, count = cls.get(search)
        node_schema = CollectorsNodePresentationSchema(many=True)
        items = node_schema.dump(nodes)

        return {"total_count": count, "items": items}

    @classmethod
    def add_new(cls, node_data):
        new_node_schema = NewCollectorsNodeSchema()
        node = new_node_schema.load(node_data)
        db.session.add(node)
        db.session.commit()
        return node

    @classmethod
    def update(cls, node_id, node_data):
        new_node_schema = NewCollectorsNodeSchema()
        updated_node = new_node_schema.load(node_data)
        node = cls.query.get(node_id)
        node = updated_node

        db.session.commit()
        return node

    @classmethod
    def delete(cls, node_id):
        node = cls.query.get(node_id)
        db.session.delete(node)
        db.session.commit()



from marshmallow import post_load

from core.managers.db_manager import db
from shared.schema.parameter import ParameterType, ParameterSchema


class NewParameterSchema(ParameterSchema):
    @post_load
    def make_parameter(self, data, **kwargs):
        return Parameter(**data)


class Parameter(db.Model):
    key = db.Column(db.String(), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    type = db.Column(db.Enum(ParameterType))

    def __init__(self, key, name, description, type):
        self.key = key
        self.name = name
        self.description = description
        self.type = type

    @classmethod
    def add(cls, data):
        if cls.get_by_key(data["key"]):
            return None
        schema = NewParameterSchema()
        param = schema.load(data)
        db.session.add(param)
        db.session.commit()

    @classmethod
    def find_by_key(cls, key):
        return cls.query.get(key)

    @classmethod
    def filter_by_key(cls, key):
        return cls.query.filter_by(key=key).first()

    @classmethod
    def get_by_key(cls, key):
        param = cls.query.get(key)
        return ParameterSchema().dump(param) if param else None

    @classmethod
    def get_all_json(cls):
        param, count = cls.query.all(), cls.query.count()
        schema = ParameterSchema(many=True)
        items = schema.dump(param)

        return {"total_count": count, "items": items}



parameters = [
    {
        "name": "Feed URL",
        "description": "Full url for RSS feed",
        "key": "FEED_URL",
        "type": "STRING",
    },
    {
        "name": "User agent",
        "description": "Type of user agent",
        "key": "USER_AGENT",
        "type": "STRING",
    },
    {
        "name": "Content Location",
        "description": "Location of the content Field",
        "key": "CONTENT_LOCATION",
        "type": "STRING",
    },
    {
        "name": "Proxy server",
        "description": "Type SOCKS5 proxy server as username:password@ip:port or ip:port",
        "key": "PROXY_SERVER",
        "type": "STRING",
    },
    {
        "name": "Refresh interval in minutes",
        "description": "How often this should be triggerd",
        "key": "REFRESH_INTERVAL",
        "type": "NUMBER",
    },
    {
        "name": "Email server type",
        "description": "Server type parameter means IMAP or POP3 email server",
        "key": "EMAIL_SERVER_TYPE",
        "type": "STRING",
    },
    {
        "name": "Email server hostname",
        "description": "Hostname of email server",
        "key": "EMAIL_SERVER_HOSTNAME",
        "type": "STRING",
    },
    {
        "name": "Email server port",
        "description": "Port of email server",
        "key": "EMAIL_SERVER_PORT",
        "type": "NUMBER",
    },
    {
        "name": "Username",
        "description": "Username of email account",
        "key": "EMAIL_USERNAME",
        "type": "STRING",
    },
    {
        "name": "Password",
        "description": "Password of email account",
        "key": "EMAIL_PASSWORD",
        "type": "STRING",
    },
    {
        "name": "Slack API token",
        "description": "API token for Slack authentication.",
        "key": "SLACK_API_TOKEN",
        "type": "STRING",
    },
    {
        "name": "Collected workspaces channels ID",
        "description": "Channels which will be collected.",
        "key": "WORKSPACE_CHANNELS_ID",
        "type": "STRING",
    },
    {
        "name": "Twitter API key",
        "description": "API key of Twitter account",
        "key": "TWITTER_API_KEY",
        "type": "STRING",
    },
    {
        "name": "Twitter API key secret",
        "description": "API key secret of Twitter account",
        "key": "TWITTER_API_KEY_SECRET",
        "type": "STRING",
    },
    {
        "name": "Twitter access token",
        "description": "Twitter access token of Twitter account",
        "key": "TWITTER_ACCESS_TOKEN",
        "type": "STRING",
    },
    {
        "name": "Twitter access token secret",
        "description": "Twitter access token secret of Twitter account",
        "key": "TWITTER_ACCESS_TOKEN_SECRET",
        "type": "STRING",
    },
    {
        "name": "Search by keywords",
        "description": "Search tweets bykeywords",
        "key": "SEARCH_KEYWORDS",
        "type": "STRING",
    },
    {
        "name": "Search by hashtags",
        "description": "Search tweets by hashtags",
        "key": "SEARCH_HASHTAGS",
        "type": "STRING",
    },
    {
        "description": "How many tweets will be provided",
        "name": "Number of tweets",
        "key": "NUMBER_OF_TWEETS",
        "type": "NUMBER",
    },
    {
        "description": "Full url for web pageor folder of html file",
        "name": "Web URL",
        "key": "WEB_URL",
        "type": "STRING",
    },
    {
        "description": "Name of webdriver for Selenium (chrome|firefox)",
        "name": "Name of Webdriver",
        "key": "WEBDRIVER",
        "type": "STRING",
    },
    {
        "description": "Using Tor service (yes|no)",
        "name": "Do you want to use Tor service? Enter Yes or No",
        "key": "TOR",
        "type": "STRING",
    },
    {
        "description": "Username for authentication with basic auth header",
        "name": "Username for web page authentication",
        "key": "AUTH_USERNAME",
        "type": "STRING",
    },
    {
        "description": "Password for authentication with basic auth header",
        "name": "Password for web page authentication",
        "key": "AUTH_PASSWORD",
        "type": "STRING",
    },
    {
        "description": "PATH to clients certificates directory",
        "name": "PATH to directory with clients certificates",
        "key": "CLIENT_CERT_DIR",
        "type": "STRING",
    },
    {
        "description": "This is a selector of the clickable element to close the popup",
        "name": "SELECTOR at TITLE PAGE: Popup removal",
        "key": "POPUP_CLOSE_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "This is a selector of the clickable element to load the next page",
        "name": "SELECTOR at TITLE PAGE: Next page",
        "key": "NEXT_BUTTON_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "This is a selector of the clickable element to load more content",
        "name": "SELECTOR at TITLE PAGE: Load more",
        "key": "LOAD_MORE_BUTTON_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "If pagination or progressive loading, maximum number of pages to visit. Default: 1",
        "name": "Pagination limit",
        "key": "PAGINATION_LIMIT",
        "type": "NUMBER",
    },
    {
        "description": "Selector that matches the link to the article. Matching results should contain a href attribute.",
        "name": "SELECTOR at TITLE PAGE: Links to articles",
        "key": "SINGLE_ARTICLE_LINK_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "Selector for article title",
        "name": "SELECTOR at ARTICLE: Article title",
        "key": "TITLE_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "OPTIONAL: Selector of article description or summary",
        "name": "SELECTOR at ARTICLE: short summary",
        "key": "ARTICLE_DESCRIPTION_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "Selector for the article content / text of the article",
        "name": "SELECTOR at ARTICLE: Article content",
        "key": "ARTICLE_FULL_TEXT_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "OPTIONAL: Selector to find the author of the post",
        "name": "SELECTOR at ARTICLE: Author",
        "key": "AUTHOR_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "OPTIONAL: Selector of the published date",
        "name": "SELECTOR at ARTICLE: Date published",
        "key": "PUBLISHED_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "OPTIONAL: Selector for links to article attachments",
        "name": "SELECTOR at ARTICLE: Attachment selector",
        "key": "ATTACHMENT_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "Collect only first few words of the article (perhaps for legal reasons)",
        "name": "Limit article body to this many words",
        "key": "WORD_LIMIT",
        "type": "STRING",
    },
    {
        "description": "OPTIONAL: Selector of an additional article ID",
        "name": "SELECTOR at ARTICLE: Additional ID selector",
        "key": "ADDITIONAL_ID_SELECTOR",
        "type": "STRING",
    },
    {
        "description": "Full url for Atom feed",
        "name": "Atom feed URL",
        "key": "ATOM_FEED_URL",
        "type": "STRING",
    },
    {
        "name": "Source Group",
        "key": "SOURCE_GROUP",
        "description": "OSINT Source group to inspect",
        "type": "STRING",
    },
    {
        "name": "Keywords",
        "key": "KEYWORDS",
        "description": "Keywords to Tag on seperated by ','",
        "type": "STRING",
    },
    {
        "name": "Regular Expression",
        "key": "REGULAR_EXPRESSION",
        "description": "Regular expression for items matching",
        "type": "STRING",
    },
    {
        "name": "Attribute name",
        "key": "ATTRIBUTE_NAME",
        "description": "Name of attribute for extracted data",
        "type": "STRING",
    },
    {
        "name": "language",
        "key": "LANGUAGE",
        "description": "language",
        "type": "STRING",
    },
    {
        "name": "Header template path",
        "description": "Path of header template file",
        "key": "HEADER_TEMPLATE_PATH",
        "type": "STRING",
    },
    {
        "name": "Body template path",
        "description": "Path of body template file",
        "key": "BODY_TEMPLATE_PATH",
        "type": "STRING",
    },
    {
        "name": "Footer template path",
        "description": "Path of footer template file",
        "key": "FOOTER_TEMPLATE_PATH",
        "type": "STRING",
    },
    {
        "name": "HTML template with its path",
        "description": "Path of html template file",
        "key": "HTML_TEMPLATE_PATH",
        "type": "STRING",
    },
    {
        "name": "TEXT template with its path",
        "description": "Path of text template file",
        "key": "TEXT_TEMPLATE_PATH",
        "type": "STRING",
    },
    {
        "name": "MISP template with its path",
        "description": "Path of MISP template file",
        "key": "MISP_TEMPLATE_PATH",
        "type": "STRING",
    },
    {
        "name": "FTP URL",
        "description": "FTP server url",
        "key": "FTP_URL",
        "type": "STRING",
    },
    {
        "description": "SMTP server for sending emails",
        "key": "SMTP_SERVER",
        "name": "SMTP server",
        "type": "STRING",
    },
    {
        "description": "SMTP server port for sending emails",
        "key": "SMTP_SERVER_PORT",
        "name": "SMTP server port",
        "type": "STRING",
    },
    {
        "description": "Username for email account",
        "key": "EMAIL_USERNAME",
        "name": "Email username",
        "type": "STRING",
    },
    {
        "description": "Password for email account",
        "key": "EMAIL_PASSWORD",
        "name": "Email password",
        "type": "STRING",
    },
    {
        "description": "Email address of recipient",
        "key": "EMAIL_RECIPIENT",
        "name": "Email recipient",
        "type": "STRING",
    },
    {
        "description": "Text of email subject",
        "key": "EMAIL_SUBJECT",
        "name": "Email subject",
        "type": "STRING",
    },
    {
        "name": "Email message",
        "description": "Text of email message",
        "key": "EMAIL_MESSAGE",
        "type": "STRING",
    },
    {
        "name": "Do you want use email encrypt (yes/no)",
        "description": "Turn ON/OFF email encryption",
        "key": "EMAIL_ENCRYPTION",
        "type": "STRING",
    },
    {
        "name": "Wordpress URL address",
        "description": "URL address of wordpress webpage",
        "key": "WP_URL",
        "type": "STRING",
    },
    {
        "name": "Username of wordpress editor",
        "description": "Post editor's username",
        "key": "WP_USER",
        "type": "STRING",
    },
    {
        "name": "Secret key of application",
        "description": "Secret key created in Wordpress for Python application",
        "key": "WP_PYTHON_APP_SECRET",
        "type": "STRING",
    },
    {
        "name": "MISP url",
        "description": "MISP server https url",
        "key": "MISP_URL",
        "type": "STRING",
    },
    {
        "name": "MISP API key",
        "description": "User MISP API key",
        "key": "MISP_API_KEY",
        "type": "STRING",
    },
]


collectors = [
    {
        "description": "Collector for gathering data from RSS feeds",
        "name": "RSS Collector",
        "parameters": [
            "FEED_URL",
            "USER_AGENT",
            "CONTENT_LOCATION",
            "PROXY_SERVER",
            "REFRESH_INTERVAL",
        ],
        "type": "RSS_COLLECTOR",
    },
    {
        "description": "Collector for gathering data from emails",
        "name": "EMAIL Collector",
        "parameters": [
            "EMAIL_SERVER_TYPE",
            "EMAIL_SERVER_HOSTNAME",
            "EMAIL_SERVER_PORT",
            "EMAIL_USERNAME",
            "EMAIL_PASSWORD",
            "PROXY_SERVER",
            "REFRESH_INTERVAL",
        ],
        "type": "EMAIL_COLLECTOR",
    },
    {
        "description": "Collector for gathering data from Slack",
        "name": "Slack Collector",
        "parameters": [
            "SLACK_API_TOKEN",
            "WORKSPACE_CHANNELS_ID",
            "PROXY_SERVER",
            "REFRESH_INTERVAL",
        ],
        "type": "SLACK_COLLECTOR",
    },
    {
        "description": "Collector for gathering data from Twitter",
        "name": "Twitter Collector",
        "parameters": [
            "TWITTER_API_KEY",
            "TWITTER_API_KEY_SECRET",
            "TWITTER_ACCESS_TOKEN",
            "TWITTER_ACCESS_TOKEN_SECRET",
            "PROXY_SERVER",
            "REFRESH_INTERVAL",
        ],
        "type": "TWITTER_COLLECTOR",
    },
    {
        "description": "Collector for gathering data from web page",
        "name": "Web Collector",
        "parameters": [
            "WEB_URL",
            "WEBDRIVER",
            "TOR",
            "USER_AGENT",
            "PROXY_SERVER",
            "REFRESH_INTERVAL",
            "AUTH_USERNAME",
            "AUTH_PASSWORD",
            "CLIENT_CERT_DIR",
            "POPUP_CLOSE_SELECTOR",
            "TITLE_SELECTOR",
            "NEXT_BUTTON_SELECTOR",
            "LOAD_MORE_BUTTON_SELECTOR",
            "PAGINATION_LIMIT",
            "ARTICLE_DESCRIPTION_SELECTOR",
            "ARTICLE_FULL_TEXT_SELECTOR",
            "SINGLE_ARTICLE_LINK_SELECTOR",
            "AUTHOR_SELECTOR",
            "PUBLISHED_SELECTOR",
            "ATTACHMENT_SELECTOR",
            "WORD_LIMIT",
            "ADDITIONAL_ID_SELECTOR",
        ],
        "type": "WEB_COLLECTOR",
    },
    {
        "description": "Collector for gathering data from Atom feeds",
        "name": "Atom Collector",
        "parameters": [
            "ATOM_FEED_URL",
            "USER_AGENT",
            "PROXY_SERVER",
            "REFRESH_INTERVAL",
        ],
        "type": "ATOM_COLLECTOR",
    },
    {
        "description": "Collector for manual input of news items",
        "name": "Manual Collector",
        "parameters": [
            "PROXY_SERVER",
            "REFRESH_INTERVAL",
        ],
        "type": "MANUAL_COLLECTOR",
    },
]

bots = [
    {
        "type": "ANALYST_BOT",
        "name": "Analyst Bot",
        "parameter_values": [
            {
                "value": "",
                "parameter": "REGULAR_EXPRESSION",
            },
            {
                "value": "",
                "parameter": "ATTRIBUTE_NAME",
            },
            {
                "value": "",
                "parameter": "SOURCE_GROUP",
            },
            {
                "value": "",
                "parameter": "REFRESH_INTERVAL",
            },
        ],
        "description": "Bot for news items analysis",
    },
    {
        "type": "GROUPING_BOT",
        "name": "Grouping Bot",
        "parameter_values": [
            {"value": "", "parameter": "SOURCE_GROUP"},
            {"value": "", "parameter": "REGULAR_EXPRESSION"},
            {"value": "", "parameter": "REFRESH_INTERVAL"},
        ],
        "description": "Bot for grouping news items into aggregates",
    },
    {
        "type": "NLP_BOT",
        "name": "NLP Bot",
        "parameter_values": [
            {"value": "", "parameter": "SOURCE_GROUP"},
            {"value": "", "parameter": "LANGUAGE"},
            {"value": "", "parameter": "REFRESH_INTERVAL"},
        ],
        "description": "Bot for naturale language processing of news items",
    },
    {
        "type": "TAGGING_BOT",
        "name": "Tagging Bot",
        "parameter_values": [
            {"value": "", "parameter": "SOURCE_GROUP"},
            {"value": "", "parameter": "KEYWORDS"},
            {"value": "", "parameter": "REFRESH_INTERVAL"},
        ],
        "description": "Bot for tagging news items",
    },
]


presenters = [
    {
        "type": "PDF_PRESENTER",
        "description": "Presenter for generating PDF documents",
        "parameters": ["HEADER_TEMPLATE_PATH", "BODY_TEMPLATE_PATH", "FOOTER_TEMPLATE_PATH"],
        "name": "PDF Presenter",
    },
    {
        "type": "HTML_PRESENTER",
        "name": "HTML Presenter",
        "description": "Presenter for generating html documents",
        "parameters": ["HTML_TEMPLATE_PATH"],
    },
    {
        "type": "TEXT_PRESENTER",
        "name": "TEXT Presenter",
        "description": "Presenter for generating text documents",
        "parameters": ["TEXT_TEMPLATE_PATH"],
    },
    {
        "type": "MISP_PRESENTER",
        "name": "MISP Presenter",
        "description": "Presenter for generating MISP platform",
        "parameters": ["MISP_TEMPLATE_PATH"],
    },
]

publishers = [
    {
        "type": "FTP_PUBLISHER",
        "name": "FTP Publisher",
        "description": "Publisher for publishing to FTP server",
        "parameters": ["FTP_URL"],
    },
    {
        "type": "EMAIL_PUBLISHER",
        "name": "EMAIL Publisher",
        "description": "Publisher for publishing by email",
        "parameters": [
            "SMTP_SERVER",
            "SMTP_SERVER_PORT",
            "EMAIL_USERNAME",
            "EMAIL_PASSWORD",
            "EMAIL_RECIPIENT",
            "EMAIL_SUBJECT",
            "EMAIL_MESSAGE",
            "EMAIL_ENCRYPTION",
        ],
    },
    {
        "type": "TWITTER_PUBLISHER",
        "name": "Twitter Publisher",
        "description": "Publisher for publishing to Twitter account",
        "parameters": ["TWITTER_API_KEY", "TWITTER_API_KEY_SECRET", "TWITTER_ACCESS_TOKEN", "TWITTER_ACCESS_TOKEN_SECRET"],
    },
    {
        "type": "WORDPRESS_PUBLISHER",
        "name": "Wordpress Publisher",
        "description": "Publisher for publishing on Wordpress webpage",
        "parameters": ["WP_URL", "WP_USER", "WP_PYTHON_APP_SECRET"],
    },
    {
        "type": "MISP_PUBLISHER",
        "name": "MISP Publisher",
        "description": "Publisher for publishing in MISP",
        "parameters": ["MISP_URL", "MISP_API_KEY"],
    },
]



def test_api_key():
    from core.config import Config

    api_key = Config.API_KEY
    assert api_key == "test_key"


def test_flask_secret_key(app):
    secret_key = app.config.get("SECRET_KEY", None)
    assert secret_key == "test_key"



def test_is_alive(client):
    response = client.get("/api/v1/isalive")
    assert b'"isalive": true' in response.data


def test_is_alive_fail(client):
    response = client.get("/api/v1/isalive")
    assert b'"isalive": false' not in response.data


def test_auth_login_fail(client):
    response = client.get("/api/v1/auth/login")
    assert response.status_code == 401


def test_auth_logout(client):
    response = client.get("/api/v1/auth/logout")
    assert response.status_code == 200


def test_auth_login(client):
    body = {"username": "admin", "password": "admin"}
    response = client.post("/api/v1/auth/login", json=body)
    assert response.status_code == 200


def test_access_token(access_token):
    assert access_token is not None


def test_user_profile(client, auth_header):
    response = client.get("/api/v1/users/my-profile", headers=auth_header)
    assert response.json
    assert response.data
    assert response.status_code == 200



from flask_restful import Api
from core.config import Config
from swagger_ui import api_doc
import os

import core.api as core_api


def initialize(app):
    api = Api(app)

    openapi_yaml = os.path.join("core", Config.OpenAPI, "openapi3_0.yaml")
    api_doc(app, config_path=openapi_yaml, url_prefix="/api/v1/doc", editor=False)

    core_api.analyze.initialize(api)
    core_api.assess.initialize(api)
    core_api.assets.initialize(api)
    core_api.auth.initialize(api)
    core_api.bots.initialize(api)
    core_api.collectors.initialize(api)
    core_api.config.initialize(api)
    core_api.dashboard.initialize(api)
    core_api.isalive.initialize(api)
    core_api.publish.initialize(api)
    core_api.user.initialize(api)
    core_api.remote.initialize(api)



from flask import request
import json
from flask_restful import Resource, reqparse
from datetime import datetime, timedelta

from core.managers import sse_manager, bots_manager
from core.managers.log_manager import logger
from core.managers.auth_manager import api_key_required
from core.model import news_item, word_list, bots_node, bot


class BotGroupAction(Resource):
    @api_key_required
    def put(self):
        aggregate_ids = request.json
        if not aggregate_ids:
            return {"No aggregate ids provided"}, 400
        response, code = news_item.NewsItemAggregate.group_aggregate(aggregate_ids)
        sse_manager.news_items_updated()
        return response, code


class BotUnGroupAction(Resource):
    @api_key_required
    def put(self):
        newsitem_ids = request.json
        if not newsitem_ids:
            return {"No aggregate ids provided"}, 400
        response, code = news_item.NewsItemAggregate.ungroup_aggregate(newsitem_ids)
        sse_manager.news_items_updated()
        return response, code


class NewsItemData(Resource):
    @api_key_required
    def get(self):
        try:
            limit = request.args.get("limit", default=(datetime.now() - timedelta(weeks=1)).isoformat())
            return news_item.NewsItemData.get_all_news_items_data(limit)
        except Exception:
            logger.log_debug_trace("GET /api/v1/bots/news-item-data failed")
            return "", 400


class UpdateNewsItemData(Resource):
    @api_key_required
    def put(self, news_item_data_id):
        try:
            if not request.json:
                return {"Not update data provided"}
            if language := request.json.get("language"):
                return news_item.NewsItemData.update_news_item_lang(news_item_data_id, language)
            return {"Not implemented"}
        except Exception:
            logger.log_debug_trace("GET /api/v1/bots/news-item-data failed")
            return "", 400


class BotsNode(Resource):
    @api_key_required
    def get(self, node_id):
        return bots_node.BotsNode.get_json_by_id(node_id)

    @api_key_required
    def put(self, node_id):
        return bots_manager.update_bots_node(node_id, request.json)

    @api_key_required
    def post(self):
        return bots_manager.add_bots_node(request.json)

    @api_key_required
    def delete(self, node_id):
        bots_node.BotsNode.delete(node_id)


class UpdateNewsItemAttributes(Resource):
    @api_key_required
    def put(self, news_item_data_id):
        news_item.NewsItemData.update_news_item_attributes(news_item_data_id, request.json)


class UpdateNewsItemTags(Resource):
    @api_key_required
    def put(self, aggregate_id):
        news_item.NewsItemAggregate.update_tags(aggregate_id, request.json)


class UpdateNewsItemsAggregateSummary(Resource):
    @api_key_required
    def put(self, aggregate_id):
        news_item.NewsItemAggregate.update_news_items_aggregate_summary(aggregate_id, request.json)


class GetNewsItemsAggregate(Resource):
    @api_key_required
    def get(self, group_id):
        limit = request.args.get("limit", "")
        resp_str = news_item.NewsItemAggregate.get_news_items_aggregate(group_id, limit)
        return json.loads(resp_str)


class GetDefaultNewsItemsAggregate(Resource):
    @api_key_required
    def get(self):
        limit = request.args.get("limit", "")
        resp_str = news_item.NewsItemAggregate.get_default_news_items_aggregate(limit)
        return json.loads(resp_str)


class WordListEntries(Resource):
    @api_key_required
    def delete(self, word_list_id, entry_name):
        return word_list.WordListEntry.delete_entries(word_list_id, entry_name)

    @api_key_required
    def put(self, word_list_id, entry_name):
        return word_list.WordListEntry.update_word_list_entries(word_list_id, request.json)


class BotsInfo(Resource):
    def get(self):
        search = request.args.get(key="search", default=None)
        return bot.Bot.get_all_json(search)


class BotInfo(Resource):
    def get(self, bot_id):
        return bot.Bot.get_by_id(bot_id)

    def put(self, bot_id):
        return bot.Bot.update_bot_parameters(bot_id, request.json)


def initialize(api):
    api.add_resource(BotsInfo, "/api/v1/bots")
    api.add_resource(BotInfo, "/api/v1/bots/<string:bot_id>")
    api.add_resource(NewsItemData, "/api/v1/bots/news-item-data")
    api.add_resource(
        UpdateNewsItemTags,
        "/api/v1/bots/news-items-aggregate/<string:aggregate_id>/tags",
    )
    api.add_resource(
        UpdateNewsItemData,
        "/api/v1/bots/news-item-data/<string:news_item_data_id>",
    )
    api.add_resource(
        UpdateNewsItemAttributes,
        "/api/v1/bots/news-item-data/<string:news_item_data_id>/attributes",
    )
    api.add_resource(BotGroupAction, "/api/v1/bots/news-item-aggregates/group")
    api.add_resource(BotUnGroupAction, "/api/v1/bots/news-item-aggregates/ungroup")
    api.add_resource(
        GetNewsItemsAggregate,
        "/api/v1/bots/news-item-aggregates-by-group/<string:group_id>",
    )
    api.add_resource(
        UpdateNewsItemsAggregateSummary,
        "/api/v1/bots/news-items-aggregate/<string:aggregate_id>/summary",
    )
    api.add_resource(GetDefaultNewsItemsAggregate, "/api/v1/bots/news-item-aggregates")
    api.add_resource(
        WordListEntries,
        "/api/v1/bots/word-list/<int:word_list_id>/entries/<string:entry_name>",
    )
    api.add_resource(BotsNode, "/api/v1/bots/node/<string:node_id>", "/api/v1/bots/node")
